rngseed <- function(seed)
{
        seed <- as.integer(seed)
        if(seed <= 0)
                stop("Seed must be a positive integer")
        tmp <- .Fortran("rngs", seed)
        invisible()
}

##########################################################
lag <- function(x,k=1,forward=T){
  if(length(x) <=1 )return(NA)
	if(forward) {y <- c(rep(NA,k),x[1:(length(x)-k)])}
	else {y <- c(x[(1:(length(x)-k))+k],rep(NA,k))}
	y
} 

###########################################################
npsmatrix <- function(psmatrix,probs){
# finds the actual s values possible for each combination
# of d,e; also stores the length of those s's and the *normalized*
# probabilities.

  # e of 0 is 1 in array
  # e of 1 is 2 in array
  nd <- nrow(psmatrix)
  ne <- as.integer(2)
  ns <- nd+1
  dandetolength.s <- array(as.integer(0),dim=c(nd,ne))
  dandetowhat.s <- array(0.,dim=c(nd,ne,ns))
  dandetoprop.s <- array(0.,dim=c(nd,ne,ns))
  for(id in 1:nd){
    # if e=1
    showtemp <- psmatrix[id,]
    showtemp <- c(1:ns)*showtemp
    showtemp <- showtemp[showtemp>0]
    ntemp <- length(showtemp)
    dandetolength.s[id,2] <-as.integer(ntemp)
    dandetowhat.s[id,2,c(1:ntemp)] <- showtemp
    try2 <- probs[showtemp]
    try2 <- try2/sum(try2)
    try2 <- cumsum(try2)
    
    dandetoprop.s[id,2,c(1:ntemp)] <- try2
    # if e=0
    showtemp <- psmatrix[id,]
    showtemp <- 1-showtemp
    showtemp <- c(1:ns)*showtemp
    showtemp <- showtemp[showtemp>0]
    ntemp <- length(showtemp)
    dandetolength.s[id,1] <-as.integer(ntemp)
    dandetowhat.s[id,1,c(1:ntemp)] <- showtemp
    try2 <- probs[showtemp]
    try2 <- try2/sum(try2)
    try2 <- cumsum(try2)
    dandetoprop.s[id,1,c(1:ntemp)] <- try2
    
  }
  list(lengthofs=dandetolength.s,whatofs=dandetowhat.s,probofs=dandetoprop.s)}

####################################################################
# Creates the "long" or "big" data set
makebig <- function(nfix,nvary,times,tatrisk=NULL,datawide){

# inputs are: nfix,nvary,ntimes, datawide=
# id,x1,...,xnfix,[v1.1,..,vnvary.1],...,[v1.ntimes,..,vnvary.ntimes],
# d,e, and y have structure: d.1,...,d.ntimes

nsubj <- nrow(datawide)
#  Transform to long format
varying <- list(NULL)
ntimes <- length(times)
for (i in 1:(nvary+3)) {
varycol <- seq(from=nfix+(i-1)*ntimes+2,length=ntimes)
varying[[i]] <- names(datawide)[varycol]
}

v.names <- NULL

if(nvary > 0) {
for (i in 1:(nvary)){
v.names <- c(v.names,paste("v",i,sep=""))
}
}
v.names <- c(v.names,"D","E","Y")

names(datawide)[1]<- "id"

ncoldatalong <- 1+nfix+nvary+1+3
datalong <- reshape(datawide, idvar="id", varying=varying, direction="long",times=times, v.names=v.names)
datalong <- datalong[order(datalong$id,datalong$time),]
datalong <- datalong[,1:ncoldatalong]
# If first Y is NA then make it 0
# This won't happen in real data
firstNA <- seq(1,by=ntimes,length=nsubj)
datalong$Y[firstNA][is.na(datalong$Y[firstNA])] <- 0

# Calculate time at risk 
if(is.null(tatrisk)) {
tatrisk <- rep(NA,nsubj)
for (id in 1:nsubj) {
	is.id <- id==datalong$id
	tatrisk[id] <- min(ntimes,which(datalong$Y[is.id]==1),
		which(is.na(datalong$Y[is.id]))) 
	}
}

# Censoring information
datalong$C <- rep(0, nsubj*ntimes)

# Rows to be selected for "big" dataframe
select <- NULL
for (i in 1:nsubj){
select <- c(select,((i-1)*ntimes+1) : ((i-1)*ntimes + tatrisk[i]))
}

trisk <- ((1:nsubj)-1)*ntimes + tatrisk

datalong$C[trisk][is.na(datalong$Y[trisk])] <- 1 


databig <- datalong[select,]

list(databig=databig,tatrisk=tatrisk)

}

####################################################################
# Creates the full data set
makefull <- function(databig,dcol=nfix+nvary+3,ecol=nfix+nvary+4) {
ncbig <- as.integer(ncol(databig))
nrbig <- as.integer(nrow(databig))
nd <- as.integer(nlevels(as.factor(databig[,dcol])))
ne <- as.integer(nlevels(as.factor(databig[,ecol])))
ns <- as.integer(nd+1)

# define psmatrix[d,s]= E(d) for stratum s
# psmatrix  
#     [,1] [,2] [,3]
#[1,]    1    1    0
#[2,]    1    0    0

psmatrix <- matrix(1,nrow=ns,ncol=nd)

for(i in 1:nd){
  icol <- ns-i
  psmatrix[(i+1):ns,icol] <- 0
}
psmatrix <- t(psmatrix)

# define whatpsmatrix: map from d,e to s and with probs 
#   pibar[is]=preliminary estimates of Pr(S=is)
pibar <- rep(1,ns)/ns
whatpsmatrix <- npsmatrix(psmatrix,pibar)

#  define
#  nsbig(nrbig): nsbig(ibig)= number of possible s's of ibig
#  nrfull      : sum(nsbig)
#  sbig(nrbig) : sbig(ibig) a random draw from the possible s (double prec.)
temp <- .Fortran("findrfull",databig[,dcol],databig[,ecol],whatpsmatrix$lengthofs, 
whatpsmatrix$whatofs, whatpsmatrix$probofs, nrbig=nrbig,nd,ne,ns, 
nrfull=integer(1), nsbig=integer(nrbig), sbig=numeric(nrbig))
nrfull  <- as.integer(temp$nrfull)
sbig <- temp$sbig
nsbig <- as.integer(temp$nsbig)
ncbig <- as.integer(ncol(databig))
ncfull <- as.integer(ncbig+1)


# define datafull= databig whose rows are expanded by the
# possible s's. The last column has these s values.

datafull <- data.frame(.Fortran("createfull",as.matrix(databig),nrbig=nrbig,ncbig,
as.integer(dcol),as.integer(ecol),whatpsmatrix$lengthofs,nd,ne,
whatpsmatrix$whatofs,ns,datafull=matrix(0.,nrow=nrfull,ncol=ncfull),
nrfull,ncfull)$datafull)

names(datafull) <- c(names(databig),"S")                
datafull$Y[datafull$C==1] <- -999.

list(datafull=datafull,sbig=sbig, nsbig=nsbig)
}
##########################################################################
## create lags
makelagvars <- function(lag=1,idcol=1,dcol=nfix+nvary+3,ecol=nfix+nvary+4,data) {
D <- data[,dcol]  # lag 1 D
E <- data[,ecol]  # lag 1 E
emin <- min(E)
dmax <- max(D)
Dlag <- D
Elag <- E

ids <- unique(data[,idcol])
for(id in ids){        
  is.id <- id==data[,idcol]
  em1 <- lag(E[is.id],lag)
  em1[is.na(em1)] <- emin
  Elag[is.id] <- em1
  dm1 <- lag(D[is.id],lag)
  dm1[is.na(dm1)] <- dmax
  Dlag[is.id] <- dm1
 }

list(Dlag=Dlag,Elag=Elag)
}
#############################################################################
# Get initial parameters for the EM algorithm
EMstart <- function(smodel=s.fmla,ymodel=y.fmla,cmodel=c.fmla,databig,sbig){
databig <- cbind(databig,S=sbig)

model.y <- glm(ymodel,family=binomial(link=logit),data=databig[databig$C==0,])
model.c <- glm(cmodel,family=binomial(link=logit),data=databig)
model.s <- polr(smodel,data=databig)

ns <- nlevels(as.factor((sbig)))

betay <- model.y$coef
betac <- model.c$coef
zeta <- c(model.s$zeta,NA)
gamma <- -model.s$coef

delta <- log(zeta-lag(zeta))
zeta[ns] <- -999.
delta[c(1,ns)] <- -999.
theta <- c(zeta,delta,gamma,betac,betay)
theta
}
#############################################################################
# The EM algorithm
EM.ps <- function(smodel=s.fmla,ymodel=y.fmla,cmodel=c.fmla,datafull,
	theta, nsbig, rel.tol=1.e-06, abs.tol = 1.e-03, maxiter=100) {

hyfull <- model.matrix(object=ymodel,data=datafull)
nhy <- as.integer(ncol(hyfull))
hcfull <- model.matrix(object=cmodel,data=datafull)
nhc <- as.integer(ncol(hcfull))
hsfull <- model.frame(form=smodel,data=datafull)
ns <- as.integer(nlevels(as.factor((hsfull[,1]))))
hsfull <- as.matrix(hsfull[,-1])
nhs <- as.integer(ncol(hsfull))
nrfull <- as.integer(nrow(datafull))
nrbig <- as.integer(length(nsbig))

iter <- 0
likrel <- 1.0
loglikold <- -1.e06

while (likrel > rel.tol & iter < maxiter) {
iter <- iter +1 
Etemp <- .Fortran("estep",hsfull,hyfull,hcfull,datafull$S,datafull$Y,datafull$C,theta,nsbig,nrfull,nrbig,nhs,nhy,nhc,ns,pifull=rep(0.,nrfull),loglik=0.)

datafull$wts <- Etemp$pifull

loglik <- Etemp$loglik
cat(paste("Iteration - ",iter, sep=""),"  Log-likelihood:  ",Etemp$loglik,"\n")
cat("causal effect: ", theta[24],"\n")
likrel <- abs(1-loglik/loglikold)
loglikold <- loglik

model.y <- glm(y.fmla,family=binomial(link=logit),data=datafull[datafull$C==0,],weights=wts)
model.c <- glm(c.fmla,family=binomial(link=logit),data=datafull,weights=wts)
model.s <- polr(s.fmla,data=datafull,weights=wts)

betay <- model.y$coef
betac <- model.c$coef
zeta <- c(model.s$zeta,NA)
gamma <- -model.s$coef

delta <- log(zeta-lag(zeta))
zeta[ns] <- -999.
delta[c(1,ns)] <- -999.
theta <- c(zeta,delta,gamma,betac,betay)
}

# Variance-covariance matrix calculation
nvar <- as.integer(length(theta))

# Constantine's routine to calculate hessian
#  had some problems with negative diagonal entries
#hess <- .Fortran("negdel2f2",hsfull,hyfull,hcfull,datafull$S,
#	datafull$Y,datafull$C,theta,nsbig,nrfull,nrbig,nhs,nhy,nhc,ns,
#	nvar,as.integer(c(1:nvar)),varmat=matrix(0.,nrow=nvar,ncol=nvar),
#	del1=numeric(nvar),delfraction=0.005)$varmat

# Alan Genz's routine - more accurate but also requires more
#  function evaluations
hess <- .Fortran("negdel",hsfull,hyfull,hcfull,datafull$S,
	datafull$Y,datafull$C,theta,nsbig,nrfull,nrbig,nhs,nhy,nhc,ns,
	nvar,as.integer(c(1:nvar)),
	varmat=matrix(0.,nrow=nvar,ncol=nvar))$varmat

theta.out <- c(2:(ns+1),2*ns)
hess.in <- hess[-theta.out,-theta.out]
vcov <- solve(hess.in)
list(par=theta,vcov=vcov)
}
