
# load all the required functions and libraries
source("routines.r")
dyn.load("routines.dll")
require(MASS)
rngseed(12345) # Initial seed for the random numbers generation

# Step 1. get big object, databig, with columns: (id,fixed X's,time-dependent Z's,time,D,E,Y,C)
#
# Input: datawide with columns: (id,fixed X's,Z1.1,Z1.2,...,Z1.K,Z2.1,...,Zp.K,D.1,...,D.K,
#	E.1,...,E.K,Y.1,...,Y.K)

datawide <- dget("datawide")

nvary <- 0
nfix <- 5
times <- c(1:12)    

bigobj <- makebig(nfix,nvary,times,data=datawide)
databig <- bigobj$databig
databig$Y[databig$C==1] <- -999.


# Step 2. Create auxiliary variables, such as lagged distance and exposure (lag=1),
#  needed for defining models. Add these variables to databig.
# Inputs: databig, idcol, dcol, ecol

idcol <- 1  # column no. of the subject ID variable in databig
dcol <- as.integer(1+nfix+nvary+2)  # column no. of Distance variable in databig
ecol <- as.integer(dcol+1)   # column no. of Exchange variable in databig

lagvar <- makelagvars(data=databig,id=idcol,dcol=dcol,ecol=ecol)
databig$Dm1 <- lagvar$Dlag
databig$Em1 <- lagvar$Elag

# Step 3. make the "full" object, datafull.
# Inputs: databig, dcol, ecol
# In additional to dataful, other outputs are: 
# sbig: a random draw from the allowable principal strata for each subject-time unit
# nsbig: number of allowable principal strata for each subject-time unit

fullobj <- makefull(data=databig,dcol=dcol,ecol=ecol)
datafull <- fullobj$datafull
sbig <- fullobj$sbig
nsbig <- fullobj$nsbig

# Step 4. define formulas and design matrices in full dimensions.
# INPUT: formulas

y.fmla <- as.formula("Y ~ X1 + Em1 + Dm1 + time + S + E")
c.fmla <- as.formula("C ~ X1 + Em1 + Dm1 + time + S + E")
s.fmla <- as.formula("as.factor(S) ~ X1 + Em1 + Dm1 + time")

# Step 5. Obtain good initial starting values for the EM algorithm
# Input: formulas, sbig, databig

theta.0 <- EMstart(smodel=s.fmla,ymodel=y.fmla,cmodel=c.fmla,sbig,data=databig)

# Step 6. Run the EM algorithm until convergence
# Input: formulas, sbig, theta.0, nsbig, datafull, rel.tol(convergence criterion), and 
#  	maxiter(maximum number of iterations).
# Ouputs:  Converged parameter values and hessian matrix

ans <- EM.ps(smodel=s.fmla,ymodel=y.fmla,cmodel=c.fmla,theta.0,nsbig,data=datafull, rel.tol=1.e-04, maxiter=100) 
theta <- ans$par
theta[length(theta)]  # causal effect parameter
se <- sqrt(diag(ans$vcov))


###########################################################################################################

